@@ -1,3 +1,4 @@
+import datetime
 import random
 
 import requests
@@ -10,7 +11,7 @@ from .stream import Stream
 
 class Client:
 
-    def __init__(self, system_url='http://localhost:9131/access', stream_url='ws://localhost:9131/stream', app_url='http://localhost:9150'):
+    def __init__(self, system_url='https://api.vyze.io/system/', stream_url='ws://api.vyze.io/stream/', app_url='https://api.vyze.io/app/'):
         self.__app_url = app_url
         self.__stream_url = stream_url
         self.__system_url = system_url
@@ -30,7 +31,7 @@ class Client:
         self.__origins_cache = {}
 
     def login(self, username, password):
-        resp = requests.post(f'{self.__app_url}/user/login', json={
+        resp = requests.post(f'{self.__app_url}user/login', json={
             'username': username,
             'password': password,
         })
@@ -46,13 +47,13 @@ class Client:
         return True
 
     def resolve_universe(self, name):
-        return self.__get(f'{self.__app_url}/universe/resolve/{name}')
+        return self.__get(f'{self.__app_url}universe/resolve/{name}')
 
     def use_universe(self, name):
         universe_id = self.resolve_universe(name)
         if not universe_id:
             raise RuntimeError('invalid universe')
-        self.__universe = self.__get(f'{self.__app_url}/universe/{universe_id}')
+        self.__universe = self.__get(f'{self.__app_url}universe/{universe_id}')
         if not self.__universe:
             raise RuntimeError('could not resolve universe')
         self.__space = self.__universe['space']
@@ -76,7 +77,7 @@ class Client:
 
         obj = self.__resolve_cache.get(identifier)
         if obj is None:
-            resp = requests.get(f'{self.__app_url}/universe/resolve', params={'i': identifier})
+            resp = requests.get(f'{self.__app_url}universe/resolve', params={'i': identifier})
             obj = resp.json()
             self.__resolve_cache[identifier] = obj
 
@@ -88,59 +89,64 @@ class Client:
                 raise RuntimeError('no universe or space selected')
             space = self.__space
 
-        return self.__post(f'{self.__system_url}/objects', {
+        return self.__post(f'{self.__system_url}objects', {
             'abstracts': self.resolve(abstracts),
             'space': space,
             'name': name
         })['object']
 
-    def create_relation(self, origin_id: str, target_id: str, abstracts: List[str], name='', space=None):
+    def create_relation(self, origin_id: str, target_id: str, abstracts: List[str], name='', space: str | None = None, time: datetime.datetime | None = None):
         if not space:
             if not self.__space:
                 raise RuntimeError('no universe or space selected')
             space = self.__space
 
-        return self.__post(f'{self.__system_url}/relations', {
+        body = {
             'origin': origin_id,
             'target': target_id,
             'abstracts': self.resolve(abstracts),
             'space': space,
             'name': name
-        })['relation']
+        }
+
+        if time is not None:
+            body['time'] = time.isoformat() + 'Z'
+
+        return self.__post(f'{self.__system_url}relations', body)['relation']
 
     def get_object(self, id: str):
-        return self.__get(f'{self.__system_url}/objects/{id}')['object']
+        return self.__get(f'{self.__system_url}objects/{self.resolve(id)}')['object']
 
     def delete_object(self, id: str):
-        return self.__delete(f'{self.__system_url}/objects/{id}')
+        return self.__delete(f'{self.__system_url}objects/{self.resolve(id)}')
 
     def set_name(self, id: str, name: str):
-        self.__post(f'{self.__system_url}/objects/{id}/name', {
+        self.__post(f'{self.__system_url}objects/{self.resolve(id)}/name', {
             'name': name,
         })
 
     def get_data(self, id: str) -> bytes:
-        return self.__get(f'{self.__system_url}/objects/{id}/data', is_json=False)
+        return self.__get(f'{self.__system_url}objects/{self.resolve(id)}/data', is_json=False)
 
     def set_data(self, id: str, data: bytes):
-        self.__post(f'{self.__system_url}/objects/{id}/data', data, is_json=False)
+        self.__post(f'{self.__system_url}objects/{self.resolve(id)}/data', data, is_json=False)
 
     def get_abstracts(self, id: str, include_self=False, include_direct=True, include_transitive=False):
-        return self.__get_hierarchy(id, include_self, include_direct, include_transitive, 'abstracts', self.__abstract_cache)
+        return self.__get_hierarchy(self.resolve(id), include_self, include_direct, include_transitive, 'abstracts', self.__abstract_cache)
 
     def get_specials(self, id: str, include_self=False, include_direct=True, include_transitive=False):
-        return self.__get_hierarchy(id, include_self, include_direct, include_transitive, 'specials', self.__special_cache)
+        return self.__get_hierarchy(self.resolve(id), include_self, include_direct, include_transitive, 'specials', self.__special_cache)
 
     def get_targets(self, id: str) -> dict:
-        return self.__get_relations(id, 'targets', self.__targets_cache)
+        return self.__get_relations(self.resolve(id), 'targets', self.__targets_cache)
 
     def get_origins(self, id: str) -> dict:
-        return self.__get_relations(id, 'origins', self.__origins_cache)
+        return self.__get_relations(self.resolve(id), 'origins', self.__origins_cache)
 
     def set_value(self, id: str, field: str, value):
         self.add_value(id, field, value)
 
-    def add_value(self, id: str, field: str, value):
+    def add_value(self, id: str, field: str, value, time=None):
         field_id = self.resolve(field)
         targets = self.get_targets(field_id)
         target_id = targets['ffffffffffffffffffffffffffffffff']
@@ -153,7 +159,8 @@ class Client:
             data = time_to_raw(value)
         target = self.create_object([target_id])
         self.set_data(target['id'], data)
-        return self.create_relation(id, target['id'], [field_id])
+        r = self.create_relation(id, target['id'], [field_id], time=time)
+        return r
 
     def get_value(self, id: str, field: str) -> any:
         vals = self.get_values(id, field)
@@ -190,12 +197,12 @@ class Client:
         return Stream(self, self.__stream_url)
 
     def query(self, query, valuation: Dict[str, dict]) -> List[any] | None:
-        return self.__post(f'{self.__system_url}/text_query', {
+        return self.__post(f'{self.__system_url}text_query', {
             'query': query,
             'valuation': valuation,
         })['valuations']
 
-    def __get_values_query(self, field_id: str, name: str, time=None, sampling=None) -> Tuple[str, str]:
+    def __get_values_query(self, field_id: str, name: str) -> Tuple[str, str]:
         targets = self.get_targets(field_id)
         target_id = targets['ffffffffffffffffffffffffffffffff']
         data = self.__get_type(target_id)
@@ -222,7 +229,7 @@ class Client:
         hier = cache.get(ck)
         if not hier:
             hier = self.__get(
-                f'{self.__system_url}/objects/{id}/{name}'
+                f'{self.__system_url}objects/{id}/{name}'
                 f'?self={"1" if include_self else "0"}'
                 f'&direct={"1" if include_direct else "0"}'
                 f'&transitive={"1" if include_transitive else "0"}')['ids']
@@ -234,7 +241,7 @@ class Client:
         rel = cache.get(ck)
         if not rel:
             rel = self.__get(
-                f'{self.__system_url}/objects/{id}/{name}')['idPairs']
+                f'{self.__system_url}objects/{id}/{name}')['idPairs']
             cache[ck] = rel
         return rel
 
@@ -281,7 +288,7 @@ class Client:
         return None
 
     def __refresh(self):
-        resp = requests.post(f'{self.__app_url}/user/refresh', json={
+        resp = requests.post(f'{self.__app_url}user/refresh', json={
             'userId': self.__user_id,
             'refreshToken': self.__refresh_token,
         })
@@ -308,7 +315,7 @@ class Client:
 
         perms = int(perm_reqs[1], 16)
 
-        resp = self.__get(f'{self.__app_url}/permission/{perm_reqs[0]}')
+        resp = self.__get(f'{self.__app_url}permission/{perm_reqs[0]}')
         if (perms & resp['perms']) != perms:
             return False
 
