@@ -1,7 +1,7 @@
 from src import vyze
 
 if __name__ == "__main__":
-    client = vyze.Client()
+    client = vyze.Client(system_url='http://localhost:9131/access/', stream_url='ws://localhost:9131/stream/', app_url='http://localhost:9150/')
     client.login('Julian', 'goalfoev8')
     client.use_universe('geo3')
     obj = client.create_object(['base.object'], 'b2b981b1a7659088d3bcfe27b9e514fe', 'test_object')
